==> RSTouchscreenNG-0.92.2.zip: RSTouchscreenNG/projects/front-ui/src/lib/webcam.component.ts <==
/* (C) 2016-2019 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, ViewChild, ElementRef, ChangeDetectorRef, ChangeDetectionStrategy, OnDestroy, HostBinding} from '@angular/core';
import {RepetierServer, RSUtils, RepetierState, PrinterState, PrintersState} from 'binding';
import {NgRedux, select} from '@angular-redux/store';
import {SubscriptionLike} from 'rxjs';

@Component({
    selector: 'rs-webcam',
    template: `
        <div style="position: relative;height:100%" (rsClick)="toggleCam()" >
        <img class="reccam" src="/img/REC.png" *ngIf="active?.rec" title="<?php _('Capturing') ?>" style="position: absolute;width:21px;height:21px;top:5px;left:5px"><img #img [src]="url" [ngStyle]="style">
        </div>
    `,
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class RSWebcamComponent implements OnDestroy {
    @HostBinding('style.position') styPosition = 'relative';
    @HostBinding('style.height') styHeight = '100%';
    disposed = false;
    private wasDyn = false;
    url = '/img/nowebcam.png';
    @Input() dynamic = true;
    @ViewChild('img', { static: true }) img: ElementRef;
    @select('printers') printers$;
    active: PrinterState = undefined;
    style: any = {};
    lastOrient = -1;
    lastCam = -1;
    webcamId = 0;

    private subscription: SubscriptionLike;

    constructor(public server: RepetierServer,
                private redux: NgRedux<RepetierState>,
                private changeRef: ChangeDetectorRef,
                private elem: ElementRef) {
        this.subscription = this.printers$.subscribe((p: PrintersState) => {
            if (p.active) {
                this.active = p.printers[p.active];
                this.updateRotation();
            }
        });
        this.updateImg();
    }

    updateRotation = (force: boolean = false): void => {
        if (!this.active || !this.active.config || !this.active.config.webcams || !this.img) {
            if (!this.disposed) {
                setTimeout(this.updateRotation, 50);
            }
            return;
        }
        if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length) {
            const orient = this.active.config.webcams[this.webcamId].orientation;
            if (orient === this.lastOrient && !force) {
                return;
            }
            const w = this.img.nativeElement.clientWidth;
            const h = this.img.nativeElement.clientHeight;
            let trans = 'rotate(' + orient + 'deg)';
            if (orient === 90 || orient === 270) {
                trans = 'scale(' + (h / w) + ') ' + trans;
            }
            this.style = {
                '-webkit-transform': trans,
                '-ms-transform': trans,
                'transform': trans,
            };
            this.lastOrient = orient;
            this.changeRef.markForCheck();
        } else {
            const orient = 0;
            const w = this.img.nativeElement.clientWidth;
            const h = this.img.nativeElement.clientHeight;
            const trans = 'rotate(' + orient + 'deg)';
            this.style = {
                '-webkit-transform': trans,
                '-ms-transform': trans,
                'transform': trans,
            };
            this.lastOrient = orient;
            this.changeRef.markForCheck();
        }
    }

    updateUrl() {
        const lastUrl = this.url;
        // tslint:disable-next-line
        if (this.dynamic && this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 2) === 2)) {
            if (!this.wasDyn || this.webcamId !== this.lastCam) {
                this.url = '/printer/cammjpg/' + this.active.slug + '?cam=' + this.webcamId + '&rand=' + RSUtils.createAlphaID(10) + '&sess=' + encodeURIComponent(this.redux.getState().connection.session);
            }
            this.wasDyn = true;
            // tslint:disable-next-line
        } else if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 1) === 1)) {
            this.url = '/printer/camjpg/' + this.active.slug + '?cam=' + this.webcamId + '&rand=' + RSUtils.createAlphaID(10) + '&sess=' + encodeURIComponent(this.redux.getState().connection.session);
            this.wasDyn = false;
        } else {
            this.url = '/img/nowebcam.png';
            this.wasDyn = false;
        }
        this.lastCam = this.webcamId;
        if (lastUrl !== this.url) {
            this.updateRotation(true);
        }
    }

    toggleCam() {
        this.webcamId++;
        if (this.webcamId >= this.active.config.webcams.length) {
            this.webcamId = 0;
        }
        this.updateUrl();
    }

    ngOnDestroy() {
        this.url = '/img/empty.png';
        this.img.nativeElement.src = this.url; // prevent loading after removing bug
        this.disposed = true;
        this.subscription.unsubscribe();
        this.img.nativeElement.parentNode.textContent = '';
    }

    updateImg = () => {
        if (this.disposed) {
            return;
        }
        if (this.active && this.active.config && this.active.config.webcams[this.webcamId]) {
            this.updateUrl();
            setTimeout(this.updateImg, 1000 * this.active.config.webcams[this.webcamId].reloadInterval);
        } else {
            setTimeout(this.updateImg, 1000);
        }
    }
}

==> RSTouchscreenNG-0.94.3.zip: RSTouchscreenNG/projects/front-ui/src/lib/webcam.component.ts <==
/* (C) 2016-2020 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, ViewChild, ElementRef, OnDestroy, HostBinding} from '@angular/core';
import {RepetierServer, RSUtils, RepetierState, PrinterState, PrintersState} from 'binding';
import {NgRedux, select} from '@angular-redux/store';
import {SubscriptionLike} from 'rxjs';

@Component({
    selector: 'rs-webcam',
    template: `
        <div style="position: relative;height:100%" (rsClick)="toggleCam()" >
        <img class="reccam" src="/img/REC.png" *ngIf="active?.rec" title="<?php _('Capturing') ?>" style="position: absolute;width:21px;height:21px;top:5px;left:5px"><img #img [src]="url" [ngStyle]="style">
        </div>
    `
})
export class RSWebcamComponent implements OnDestroy {
    @HostBinding('style.position') styPosition = 'relative';
    @HostBinding('style.height') styHeight = '100%';
    disposed = false;
    private wasDyn = false;
    url = '/img/nowebcam.png';
    @Input() dynamic = true;
    @ViewChild('img', { static: true }) img: ElementRef;
    @select('printers') printers$;
    active: PrinterState = undefined;
    style: any = {};
    lastOrient = -1;
    lastCam = -1;
    webcamId = 0;

    private subscription: SubscriptionLike;

    constructor(public server: RepetierServer,
                private redux: NgRedux<RepetierState>,
                private elem: ElementRef) {
        this.subscription = this.printers$.subscribe((p: PrintersState) => {
            if (p.active) {
                this.active = p.printers[p.active];
                this.updateRotation();
            }
        });
        this.updateImg();
    }

    updateRotation = (force: boolean = false): void => {
        if (!this.active || !this.active.config || !this.active.config.webcams || !this.img || !this.img.nativeElement.clientWidth) {
            if (!this.disposed) {
                setTimeout(this.updateRotation, 50);
            }
            return;
        }
        if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length) {
            const orient = this.active.config.webcams[this.webcamId].orientation;
            if (orient === this.lastOrient && !force) {
                return;
            }
            console.log('updateRotation run', orient);
            const w = this.img.nativeElement.clientWidth;
            const h = this.img.nativeElement.clientHeight;
            let trans = 'rotate(' + orient + 'deg)';
            if (orient === 90 || orient === 270) {
                trans = 'scale(' + (h / w) + ') ' + trans;
            }
            this.style = {
                '-webkit-transform': trans,
                '-ms-transform': trans,
                'transform': trans,
            };
            this.lastOrient = orient;
        } else {
            const orient = 0;
            const w = this.img.nativeElement.clientWidth;
            const h = this.img.nativeElement.clientHeight;
            const trans = 'rotate(' + orient + 'deg)';
            this.style = {
                '-webkit-transform': trans,
                '-ms-transform': trans,
                'transform': trans,
            };
            this.lastOrient = orient;
        }
        console.log('new style', this.style);
    }

    updateUrl() {
        const lastUrl = this.url;
        // tslint:disable-next-line
        if (this.dynamic && this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 2) === 2)) {
            if (!this.wasDyn || this.webcamId !== this.lastCam) {
                this.url = '/printer/cammjpg/' + this.active.slug + '?cam=' + this.webcamId + '&rand=' + RSUtils.createAlphaID(10) + '&sess=' + encodeURIComponent(this.redux.getState().connection.session);
            }
            this.wasDyn = true;
            // tslint:disable-next-line
        } else if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 1) === 1)) {
            this.url = '/printer/camjpg/' + this.active.slug + '?cam=' + this.webcamId + '&rand=' + RSUtils.createAlphaID(10) + '&sess=' + encodeURIComponent(this.redux.getState().connection.session);
            this.wasDyn = false;
        } else {
            this.url = '/img/nowebcam.png';
            this.wasDyn = false;
        }
        this.lastCam = this.webcamId;
        if (lastUrl !== this.url) {
          console.log('new url');
            this.updateRotation(true);
        }
    }

    toggleCam() {
        this.webcamId++;
        if (this.webcamId >= this.active.config.webcams.length) {
            this.webcamId = 0;
        }
        this.updateUrl();
    }

    ngOnDestroy() {
        this.url = '/img/empty.png';
        this.img.nativeElement.src = this.url; // prevent loading after removing bug
        this.disposed = true;
        this.subscription.unsubscribe();
        this.img.nativeElement.parentNode.textContent = '';
    }

    updateImg = () => {
        if (this.disposed) {
            return;
        }
        if (this.active && this.active.config && this.active.config.webcams[this.webcamId]) {
            this.updateUrl();
            setTimeout(this.updateImg, 1000 * this.active.config.webcams[this.webcamId].reloadInterval);
        } else {
            setTimeout(this.updateImg, 1000);
        }
    }
}
